#include "AccelStepper.h"
#include <ArduinoOSCWiFi.h>

// Define stepper motor connections and motor interface type. Motor interface type must be set to 1 when using a driver:
#define dirPin1 0
#define stepPin1 1
#define microPin1 2
#define dirPin2 3
#define stepPin2 4
#define microPin2 5
#define dirPin3 9
#define stepPin3 10
#define microPin3 11
#define dirPin4 13
#define stepPin4 14
#define microPin4 15

#define dirPin5 28
#define stepPin5 27
#define microPin5 26

#define dirPin6 17
#define stepPin6 18
#define microPin6 16


#define accel 6000
#define maxSpeed 100 * 30


// Create a new instance of the AccelStepper class:
AccelStepper stepper1 = AccelStepper(AccelStepper::DRIVER, stepPin1, dirPin1);
AccelStepper stepper2 = AccelStepper(AccelStepper::DRIVER, stepPin2, dirPin2);
AccelStepper stepper3 = AccelStepper(AccelStepper::DRIVER, stepPin3, dirPin3);
AccelStepper stepper4 = AccelStepper(AccelStepper::DRIVER, stepPin4, dirPin4);
// AccelStepper stepper5 = AccelStepper(AccelStepper::DRIVER, stepPin5, dirPin5);
// AccelStepper stepper6 = AccelStepper(AccelStepper::DRIVER, stepPin6, dirPin6);

const char* ssid = "CBA-2.4G";
const char* pwd = "Internet";

const int recv_port = 54321;
// const int bind_port = 54345;
// const int send_port = 55555;
// const int publish_port = 54445;

int motorSpeed = 1000;  // Default speed



void onStop(const OscMessage& m) {
    Serial.println("Received /stop command");

    stepper1.stop();
    stepper2.stop();
    stepper3.stop();
    stepper4.stop();
 
}


void onStep1(const OscMessage& m){
  Serial.println("Received /step/1");
  stepper1.move(m.arg<String>(0).toInt());
}
void onStep2(const OscMessage& m){
  Serial.println("Received /step/2");
  stepper2.move(m.arg<String>(0).toInt());
}
void onStep3(const OscMessage& m){
  Serial.println("Received /step/3");
  stepper3.move(m.arg<String>(0).toInt());
}
void onStep4(const OscMessage& m){
  Serial.println("Received /step/4");
  stepper4.move(m.arg<String>(0).toInt());
}
// void onStep5(const OscMessage& m){
//   Serial.println("Received /step/5");
//   stepper5.move(m.arg<String>(0).toInt());
// }
// void onStep6(const OscMessage& m){
//   Serial.println("Received /step/6");
//   stepper6.move(m.arg<String>(0).toInt());
// }



void setup() {

   Serial.begin(115200);
    delay(2000);


    // set micropins
    pinMode(microPin1, OUTPUT);
    pinMode(microPin2, OUTPUT);
    pinMode(microPin3, OUTPUT);
    pinMode(microPin4, OUTPUT);

    digitalWrite(microPin1, HIGH); // make microstep 1/32
    digitalWrite(microPin2, HIGH);
    digitalWrite(microPin3, HIGH);
    digitalWrite(microPin4, HIGH);



// START WIFI CODE
#if defined(ESP_PLATFORM) || defined(ARDUINO_ARCH_RP2040)
#ifdef ESP_PLATFORM
    WiFi.disconnect(true, true);  // disable wifi, erase ap info
#else
    WiFi.disconnect(true);  // disable wifi
#endif
    delay(1000);
    WiFi.mode(WIFI_STA);
#endif

    WiFi.begin(ssid, pwd);

    while (WiFi.status() != WL_CONNECTED) {
        Serial.print(".");
        delay(500);
    }

    Serial.print("WiFi connected, IP = ");
    Serial.println(WiFi.localIP());
// END WIFI CODE


// stepper values START
  stepper1.setMaxSpeed(maxSpeed);
  stepper2.setMaxSpeed(maxSpeed);
  stepper3.setMaxSpeed(maxSpeed);
  stepper4.setMaxSpeed(maxSpeed);



  stepper1.setAcceleration(accel);
  stepper2.setAcceleration(accel);
  stepper3.setAcceleration(accel);
  stepper4.setAcceleration(accel);



// // stepper values END

// // OSC addresses START

    OscWiFi.subscribe(recv_port, "/stop", onStop);
    OscWiFi.subscribe(recv_port, "/step/1", onStep1);
    OscWiFi.subscribe(recv_port, "/step/2", onStep2);
    OscWiFi.subscribe(recv_port, "/step/3", onStep3);
    OscWiFi.subscribe(recv_port, "/step/4", onStep4);
 



    
    // stepper2.move(200);
    // stepper2.setSpeed(400);
    
// OSC addresses END


}

void loop() {
  // put your main code here, to run repeatedly:
   // Continuously update OSC messages
    OscWiFi.update();

       stepper1.run();
       stepper2.run();

       stepper3.run();

       stepper4.run();



}
